﻿using System;
using MVR.FileManagementSecure;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.Events;
using SimpleJSON;

namespace ky1001
{
    public class PresetLoader : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "PresetLoader";
        public string pluginVersion = "4.0";
        public string pluginDate = "2022-09-09";
        public string pluginDescription = "";
        #endregion

        private Atom _targetAtom;

        private const string _clothingPresetPath = "Custom/Atom/Person/Clothing";
        private const string _hairPresetPath = "Custom/Atom/Person/Hair";
        private const string _posePresetPath = "Custom/Atom/Person/Pose";
        private const string _skinPresetPath = "Custom/Atom/Person/Skin";
        private const int _jsonstringLimit = 15000;
        private const float _currentPathHeight = 100f;
        private const float _presetPathHeight = 230f;

        private JSONStorableBool _enableClothing;
        private JSONStorableBool _enableHair;
        private JSONStorableBool _enablePose;
        private JSONStorableBool _enableSkin;
        private JSONStorableBool _enablePreset;
        private JSONStorableBool _enableRnd;
        private JSONStorableBool _enableAutoLoad;
        private JSONStorableBool _enableSubfolder;
        private JSONStorableString _clothingPath;
        private JSONStorableString _hairPath;
        private JSONStorableString _posePath;
        private JSONStorableString _skinPath;
        private JSONStorableString _path;
        private JSONStorableString _currentClothing;
        private JSONStorableString _currentHair;
        private JSONStorableString _currentPose;
        private JSONStorableString _currentSkin;
        private JSONStorableString _current;
        private JSONStorableString _savedClothingPath;
        private JSONStorableString _savedHairPath;
        private JSONStorableString _savedPosePath;
        private JSONStorableString _savedSkinPath;
        private JSONStorableStringChooser _selectCount;
        private JSONStorableAction _setPresetPathAct;
        private List<string> _clothingPresetList = new List<string>();
        private List<string> _hairPresetList = new List<string>();
        private List<string> _posePresetList = new List<string>();
        private List<string> _skinPresetList = new List<string>();
        private List<string> _presetList = new List<string>();
        private List<string> _femaleList = new List<string>();
        private string _presetName;
        private string _presetPath;
        private string _varFileName;
        private string _varFullName;
        private string _varFullPath;
        private int _previousClothingNumber = 0;
        private int _previousHairNumber = 0;
        private int _previousPoseNumber = 0;
        private int _previousSkinNumber = 0;
        private string _previousSelectCount;
        private bool _loopStart = false;
        private int _waitCount;
        private bool _isFemale;
        private string _skinPresetFile;
        private string _loadData;
        private JSONNode _jSONNode;
        private JSONArray _asArray;
        private JSONClass _asObject;
        private string _id;

        private readonly string[] _female = new string[]
        {
            "Candy",
            "Evey",
            "Janie",
            "Kayla",
            "Lexi",
            "Maria",
            "Mia",
            "Simone",
            "Tara",
            "Tina"
        };

        public override void Init()
        {
            try
            {
                _targetAtom = containingAtom;

                if (_targetAtom.type != "Person")
                {
                    SuperController.LogError("Please attach this plugin to Person");
                    return;
                }

                var loadPresetButton = CreateButton("Load Presets");
                loadPresetButton.button.onClick.AddListener(LoadPreset);
                JSONStorableAction LoadPresetAct = new JSONStorableAction("Load Presets", LoadPreset);

                _enableAutoLoad = new JSONStorableBool("Enable Auto Load", false);
                CreateToggle(_enableAutoLoad, true);


                _enableRnd = new JSONStorableBool("Enable Random Load", true);
                var enableRnd = CreateToggle(_enableRnd);
                enableRnd.toggle.onValueChanged.AddListener(delegate { ResetCount(); });

                var sp = CreateSpacer();
                sp.height = 10f;

                _enableSubfolder = new JSONStorableBool("Include Subfolder", true);
                CreateToggle(_enableSubfolder);

                List<string> waitCount = new List<string>();
                waitCount.Add("150");
                waitCount.Add("300");
                waitCount.Add("600");
                waitCount.Add("900");
                waitCount.Add("1200");

                _selectCount = new JSONStorableStringChooser("Select wait count number", waitCount, "300", "Select wait count");
                var sc = CreatePopup(_selectCount, true);

                var sp1 = CreateSpacer(true);
                sp1.height = 45f;

                var presetName = "Clothing";
                MakeUi(presetName, SetClothingPath);
                _enableClothing = _enablePreset;
                _currentClothing = _current;
                _clothingPath = _path;
                JSONStorableAction SetClothingPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetClothingPathAct);

                presetName = "Hair";
                MakeUi(presetName, SetHairPath);
                _enableHair = _enablePreset;
                _currentHair = _current;
                _hairPath = _path;
                JSONStorableAction SetHairPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetHairPathAct);

                presetName = "Pose";
                MakeUi(presetName, SetPosePath);
                _enablePose = _enablePreset;
                _currentPose = _current;
                _posePath = _path;
                JSONStorableAction SetPosePresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetPosePathAct);

                presetName = "Skin";
                MakeUi(presetName, SetSkinPath);
                _enableSkin = _enablePreset;
                _currentSkin = _current;
                _skinPath = _path;
                JSONStorableAction SetSkinPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetSkinPathAct);

                _savedClothingPath = new JSONStorableString("Clothing Presets Path", "");
                _savedHairPath = new JSONStorableString("Hair Presets Path", "");
                _savedPosePath = new JSONStorableString("Pose Preset Path", "");
                _savedSkinPath = new JSONStorableString("Skin Preset Path", "");

                _femaleList = new List<string>(_female);

                RegisterAction(LoadPresetAct);
                RegisterAction(SetClothingPresetPathAct);
                RegisterAction(SetHairPresetPathAct);
                RegisterAction(SetPosePresetPathAct);
                RegisterAction(SetSkinPresetPathAct);
                RegisterStringChooser(_selectCount);
                RegisterBool(_enableAutoLoad);
                RegisterBool(_enableSubfolder);
                RegisterBool(_enableRnd);
                RegisterBool(_enableClothing);
                RegisterBool(_enableHair);
                RegisterBool(_enablePose);
                RegisterBool(_enableSkin);
                RegisterString(_savedClothingPath);
                RegisterString(_savedHairPath);
                RegisterString(_savedPosePath);
                RegisterString(_savedSkinPath);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void MakeUi(string presetName, UnityAction action)
        {
            var setPresetPath = CreateButton("Set " + presetName + " Presets Path");
            setPresetPath.button.onClick.AddListener(action);

            _enablePreset = new JSONStorableBool("Enable " + presetName + " Presets", false);
            CreateToggle(_enablePreset);

            _current = new JSONStorableString("", "");
            var current = CreateTextField(_current);
            current.height = _currentPathHeight;
            _path = new JSONStorableString("", "");
            var path = CreateTextField(_path, true);
            path.height = _presetPathHeight;
        }

        public void ResetCount()
        {
            _previousClothingNumber = 0;
            _previousHairNumber = 0;
            _previousPoseNumber = 0;
            _previousSkinNumber = 0;
        }

        public void LoadPreset()
        {
            int number;
            if (_enableAutoLoad.val)
            {
                _loopStart = true;
            }
            else
            {
                _loopStart = false;
                _waitCount = 0;
            }

            if (_enableClothing.val)
            {
                if (_clothingPresetList.Count != 0)
                {
                    if (_clothingPresetList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _clothingPresetList.Count);
                            }
                            while (_previousClothingNumber == number);
                            _previousClothingNumber = number;
                        }
                        else
                        {
                            number = _previousClothingNumber;
                            _previousClothingNumber = number + 1;
                            if (_previousClothingNumber >= _clothingPresetList.Count)
                            {
                                _previousClothingNumber = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val = _clothingPresetList[number];
                    _targetAtom.GetStorableByID("ClothingPresets").CallAction("LoadPreset");
                    _currentClothing.val = _clothingPresetList[number];
                }
            }

            if (_enableHair.val)
            {
                if (_hairPresetList.Count != 0)
                {
                    if (_hairPresetList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _hairPresetList.Count);
                            }
                            while (_previousHairNumber == number);
                            _previousHairNumber = number;
                        }
                        else
                        {
                            number = _previousHairNumber;
                            _previousHairNumber = number + 1;
                            if (_previousHairNumber >= _hairPresetList.Count)
                            {
                                _previousHairNumber = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("HairPresets").GetStringJSONParam("presetName").val = _hairPresetList[number];
                    _targetAtom.GetStorableByID("HairPresets").CallAction("LoadPreset");
                    _currentHair.val = _hairPresetList[number];
                }
            }

            if (_enablePose.val)
            {
                if (_posePresetList.Count != 0)
                {
                    if (_posePresetList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _posePresetList.Count);
                            }
                            while (_previousPoseNumber == number);
                            _previousPoseNumber = number;
                        }
                        else
                        {
                            number = _previousPoseNumber;
                            _previousPoseNumber = number + 1;
                            if (_previousPoseNumber >= _posePresetList.Count)
                            {
                                _previousPoseNumber = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("PosePresets").GetStringJSONParam("presetName").val = _posePresetList[number];
                    _targetAtom.GetStorableByID("PosePresets").CallAction("LoadPreset");
                    _currentPose.val = _posePresetList[number];
                }
            }

            if (_enableSkin.val)
            {
                if (_skinPresetList.Count != 0)
                {
                    if (_skinPresetList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _skinPresetList.Count);
                            }
                            while (_previousSkinNumber == number);
                            _previousSkinNumber = number;
                        }
                        else
                        {
                            number = _previousSkinNumber;
                            _previousSkinNumber = number + 1;
                            if (_previousSkinNumber >= _skinPresetList.Count)
                            {
                                _previousSkinNumber = 0;
                            }
                        }
                    }

                    var characterName = _targetAtom.GetStorableByID("geometry").GetStringChooserJSONParam("characterSelection").val;
                    if (_femaleList.Contains(characterName) || characterName.Contains("Female"))
                    {
                        _isFemale = true;
                    }
                    else
                    {
                        _isFemale = false;
                    }

                    if (_skinPresetList[number].Contains(":"))
                    {
                        var skinPresetSplit = _skinPresetList[number].Split(':');
                        _skinPresetFile = skinPresetSplit[0] + ":/" + _skinPresetPath + "/" + skinPresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _skinPresetFile = _skinPresetPath + "/" + _skinPresetList[number] + ".vap";
                    }

                    int posi = _skinPresetFile.LastIndexOf("/");
                    _skinPresetFile = _skinPresetFile.Insert(posi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_skinPresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "geometry" && _asObject["character"] != null)
                        {
                            characterName = _asObject["character"];
                        }
                    }

                    bool _isFemaleInVap = _femaleList.Contains(characterName) || characterName.Contains("Female");
                    if (_isFemale == _isFemaleInVap)
                    {
                        _targetAtom.GetStorableByID("SkinPresets").GetStringJSONParam("presetName").val = _skinPresetList[number];
                        _targetAtom.GetStorableByID("SkinPresets").CallAction("LoadPreset");
                        _currentSkin.val = _skinPresetList[number];
                    }
                    else
                    {
                        _currentSkin.val = "";
                    }
                }
            }
        }

        public void SetClothingPath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_clothingPresetPath);
            _presetPath = _clothingPresetPath;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetClothingPathAct()
        {
            _presetPath = _clothingPresetPath;
            LoadPresetsList(_savedClothingPath.val);
        }

        public void SetHairPath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_hairPresetPath);
            _presetPath = _hairPresetPath;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetHairPathAct()
        {
            _presetPath = _hairPresetPath;
            LoadPresetsList(_savedHairPath.val);
        }

        public void SetPosePath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_posePresetPath);
            _presetPath = _posePresetPath;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetPosePathAct()
        {
            _presetPath = _posePresetPath;
            LoadPresetsList(_savedPosePath.val);
        }

        public void SetSkinPath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_skinPresetPath);
            _presetPath = _skinPresetPath;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetSkinPathAct()
        {
            _presetPath = _skinPresetPath;
            LoadPresetsList(_savedSkinPath.val);
        }

        public void LoadPresetsList(string path)
        {
            if (path != null && path != "")
            {
                path = path.Replace("\\", "/");
                if (!path.Contains(_presetPath) && path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                    path = pathSplit[0] + ":/" + _presetPath;
                }

                _varFileName = "";
                if (path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                    _varFullPath = pathSplit[0];
                    var varNameSplit = _varFullPath.Split('/');
                    var varNumber = varNameSplit.Length;
                    _varFullName = varNameSplit[varNumber - 1];
                    _varFileName = _varFullName.Remove(_varFullName.Length - 4, 4);
                }

                string[] files;
                if (_enableSubfolder.val)
                {
                    ArrayList allFiles = new ArrayList();
                    GetAllFiles(path, "Preset_*.vap", ref allFiles);
                    files = (string[])allFiles.ToArray(typeof(string));
                }
                else
                {
                    files = FileManagerSecure.GetFiles(path, "Preset_*.vap");
                }

                _presetList = new List<string>();
                var displayPath = "";

                if (_varFileName == "")
                {
                    foreach (var item in files)
                    {
                        var filename = item.Replace("\\", "/");
                        filename = filename.Remove(filename.LastIndexOf("/Preset_") + 1, 7).Replace(path, "");
                        if (filename.StartsWith("/"))
                        {
                            filename = filename.Remove(0, 1);
                        }
                        filename = filename.Remove(filename.Length - 4, 4);
                        var presetPath = path.Replace(_presetPath, "");
                        if (presetPath.StartsWith("/"))
                        {
                            presetPath = presetPath.Remove(0, 1);
                        }

                        if (presetPath != "")
                        {
                            filename = presetPath + "/" + filename;
                        }
                        _presetList.Add(filename);

                        if (displayPath.Length < _jsonstringLimit)
                        {
                            displayPath = displayPath + filename + "\n";
                        }
                    }
                }
                else
                {
                    foreach (var item in files)
                    {
                        var filename = item.Replace("\\", "/");
                        filename = filename.Remove(filename.LastIndexOf("/Preset_") + 1, 7).Replace(path, "");
                        if (filename.StartsWith("/"))
                        {
                            filename = filename.Remove(0, 1);
                        }
                        filename = filename.Remove(filename.Length - 4, 4);
                        var presetPath = path.Replace(_varFullPath + ":/", "").Replace(_presetPath, "");
                        if (presetPath.StartsWith("/"))
                        {
                            presetPath = presetPath.Remove(0, 1);
                        }

                        if (presetPath != "")
                        {
                            filename = presetPath + "/" + filename;
                        }

                        filename = _varFileName + ":" + filename;
                        _presetList.Add(filename);

                        if (displayPath.Length < _jsonstringLimit)
                        {
                            displayPath = displayPath + filename + "\n";
                        }
                    }
                }

                if (_presetPath == _clothingPresetPath)
                {
                    _clothingPresetList = _presetList;
                    _clothingPath.val = displayPath;
                    _savedClothingPath.val = path;
                    _currentClothing.val = "";
                }
                else if (_presetPath == _hairPresetPath)
                {
                    _hairPresetList = _presetList;
                    _hairPath.val = displayPath;
                    _savedHairPath.val = path;
                    _currentHair.val = "";
                }
                else if (_presetPath == _posePresetPath)
                {
                    _posePresetList = _presetList;
                    _posePath.val = displayPath;
                    _savedPosePath.val = path;
                    _currentPose.val = "";
                }
                else
                {
                    _skinPresetList = _presetList;
                    _skinPath.val = displayPath;
                    _savedSkinPath.val = path;
                    _currentSkin.val = "";
                }
            }
        }

        public static void GetAllFiles(string folder, string filter, ref ArrayList allFiles)
        {
            string[] files = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(files);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                GetAllFiles(subF, filter, ref allFiles);
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                if (_savedClothingPath != null && _savedClothingPath.val != "")
                {
                    _presetPath = _clothingPresetPath;
                    _clothingPresetList = new List<string>();
                    LoadPresetsList(_savedClothingPath.val);
                }

                if (_savedHairPath != null && _savedHairPath.val != "")
                {
                    _presetPath = _hairPresetPath;
                    _hairPresetList = new List<string>();
                    LoadPresetsList(_savedHairPath.val);
                }

                if (_savedPosePath != null && _savedPosePath.val != "")
                {
                    _presetPath = _posePresetPath;
                    _posePresetList = new List<string>();
                    LoadPresetsList(_savedPosePath.val);
                }

                if (_savedSkinPath != null && _savedSkinPath.val != "")
                {
                    _presetPath = _skinPresetPath;
                    _skinPresetList = new List<string>();
                    LoadPresetsList(_savedSkinPath.val);
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                if (_previousSelectCount != _selectCount.val)
                {
                    _waitCount = 0;
                }

                if (_enableAutoLoad.val && _loopStart)
                {
                    if (_waitCount <= 0)
                    {
                        LoadPreset();
                        _waitCount = int.Parse(_selectCount.val);
                    }
                    _waitCount--;
                }

                _previousSelectCount = _selectCount.val;
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {

        }
    }
}